@@ -1,5 +1,5 @@
-# This file is part of tiny_onion, a small-as-possible solution for p2p
-# networking over tor v3 onion services.
+# This file is part of aiootp, an asynchronous pseudo-one-time-pad based
+# crypto and anonymity library.
 #
 # Licensed under the AGPLv3: https://www.gnu.org/licenses/agpl-3.0.html
 # Copyright © 2019-2021 Gonzo Investigatory Journalism Agency, LLC
@@ -16,19 +16,25 @@ __all__ = [
     "test_map_generators",
     "test_json_functions",
     "test_bytes_functions",
+    "test_token_functions",
     "test_ascii_generators",
     "__all__",
 ]
 
 
 def test_map_generators():
+    validator = StreamHMAC(key, salt=salt, pid=pid).for_encryption()
     names = keys(key, salt=salt, pid=pid).resize(64)[:6].list()
     keystream = keys(key, salt=salt)
     datastream = data(plaintext_string.encode())
-    with datastream.map_encipher(names, keystream) as encrypting:
+    cipher = datastream.map_encipher
+    with cipher(names, keystream, validator=validator) as encrypting:
         ciphertext = encrypting.dict()
         assert ciphertext
-    with pick(names, ciphertext).map_decipher(keystream.reset()) as decrypting:
+
+    validator = StreamHMAC(key, salt=salt, pid=pid).for_decryption()
+    decipher = pick(names, ciphertext).map_decipher
+    with decipher(keystream.reset(), validator=validator) as decrypting:
         plaintext = decrypting.join(b"")
         assert plaintext
     async_result = run(async_map_functions())
@@ -36,12 +42,17 @@ def test_map_generators():
 
 
 async def async_map_functions():
+    validator = StreamHMAC(key, salt=salt, pid=pid).for_encryption()
     names = await akeys(key, salt=salt, pid=pid).aresize(64)[:6].alist()
     keystream = akeys(key, salt=salt)
     datastream = adata(plaintext_string.encode())
-    async with datastream.amap_encipher(names, keystream) as encrypting:
+    cipher = datastream.amap_encipher
+    async with cipher(names, keystream, validator=validator) as encrypting:
         ciphertext = await encrypting.adict()
-    async with apick(names, ciphertext).amap_decipher(await keystream.areset()) as decrypting:
+
+    validator = StreamHMAC(key, salt=salt, pid=pid).for_decryption()
+    decipher = apick(names, ciphertext).amap_decipher
+    async with decipher(await keystream.areset(), validator=validator) as decrypting:
         plaintext = await decrypting.ajoin(b"")
     return plaintext
 
@@ -82,22 +93,59 @@ def test_bytes_functions():
     assert plaintext_of_string == run(abytes_decrypt(ciphertext_of_string, key))
 
 
+def test_token_functions():
+    ciphertext_of_bytes = pad.make_token(plaintext_bytes, pid=pid)
+    assert ciphertext_of_bytes != run(pad.amake_token(plaintext_bytes, pid=pid))
+    assert ciphertext_of_bytes != pad.make_token(plaintext_bytes)
+    assert ciphertext_of_bytes != run(pad.amake_token(plaintext_bytes))
+    assert ciphertext_of_bytes != pad.make_token(plaintext_bytes, key=salt, pid=pid)
+    assert ciphertext_of_bytes != run(pad.amake_token(plaintext_bytes, key=salt, pid=pid))
+
+    plaintext_of_bytes = pad.read_token(ciphertext_of_bytes, pid=pid)
+    assert plaintext_of_bytes == run(pad.aread_token(ciphertext_of_bytes, pid=pid))
+
+    asynchs.sleep(2)
+    try:
+        pad.read_token(ciphertext_of_bytes, pid=pid, ttl=1)
+    except TimeoutError:
+        pass
+    else:
+        raise AssertionError("Life-time for tokens is malfunctioning.")
+    try:
+        run(pad.aread_token(ciphertext_of_bytes, pid=pid, ttl=1))
+    except TimeoutError:
+        pass
+    else:
+        raise AssertionError("Life-time for tokens is malfunctioning.")
+
+
 async def async_ascii_generators():
+    validator = StreamHMAC(key, salt=salt, pid=pid).for_encryption()
     naively_padded_plaintext = 240 * "." + "testing... This." # 16 byte string + 240 == 256
-    async with adata(naively_padded_plaintext).aascii_encipher(key, salt=salt) as enciphering:
+    cipher = adata(naively_padded_plaintext).aascii_encipher
+    async with cipher(key, salt=salt, validator=validator) as enciphering:
         ciphertext = await enciphering.alist()
         assert len(ciphertext) == 1
         assert ciphertext[0].bit_length() <= 2048
-    with aunpack(ciphertext).aascii_decipher(key, salt=salt) as deciphering:
+
+    validator = StreamHMAC(key, salt=salt, pid=pid).for_decryption()
+    decipher = aunpack(ciphertext).aascii_decipher
+    async with decipher(key, salt=salt, validator=validator) as deciphering:
         deciphered = await deciphering.ajoin()
         assert deciphered == naively_padded_plaintext
 
+
+    validator = StreamHMAC(key, salt=salt, pid=pid).for_encryption()
     unpadded_plaintext = "testing..." # 10 byte string == 246 null bytes in the deciphered plaintext
-    with adata(unpadded_plaintext).aascii_encipher(key, salt=salt) as enciphering:
+    cipher = adata(unpadded_plaintext).aascii_encipher
+    async with cipher(key, salt=salt, validator=validator) as enciphering:
         ciphertext = await enciphering.alist()
         assert len(ciphertext) == 1
         assert ciphertext[0].bit_length() <= 2048
-    with aunpack(ciphertext).aascii_decipher(key, salt=salt) as deciphering:
+
+    validator = StreamHMAC(key, salt=salt, pid=pid).for_decryption()
+    decipher = aunpack(ciphertext).aascii_decipher
+    async with decipher(key, salt=salt, validator=validator) as deciphering:
         deciphered = await deciphering.ajoin()
         assert deciphered != unpadded_plaintext
         assert deciphered[246:] == unpadded_plaintext
@@ -107,21 +155,32 @@ def test_ascii_generators():
     # Plaintext is processed in 256 byte blocks. Using these lower level
     # leaves the user having to account for this. This is a very naive
     # way to handle this
+    validator = StreamHMAC(key, salt=salt, pid=pid).for_encryption()
     naively_padded_plaintext = 240 * "." + "testing... This." # 16 byte string + 240 == 256
-    with data(naively_padded_plaintext).ascii_encipher(key, salt=salt) as enciphering:
+    cipher = data(naively_padded_plaintext).ascii_encipher
+    with cipher(key, salt=salt, validator=validator) as enciphering:
         ciphertext = enciphering.list()
         assert len(ciphertext) == 1
         assert ciphertext[0].bit_length() <= 2048
-    with unpack(ciphertext).ascii_decipher(key, salt=salt) as deciphering:
+
+    validator = StreamHMAC(key, salt=salt, pid=pid).for_decryption()
+    decipher = unpack(ciphertext).ascii_decipher
+    with decipher(key, salt=salt, validator=validator) as deciphering:
         deciphered = deciphering.join()
         assert deciphered == naively_padded_plaintext
 
+
+    validator = StreamHMAC(key, salt=salt, pid=pid).for_encryption()
     unpadded_plaintext = "testing..." # 10 byte string == 246 null bytes in the deciphered plaintext
-    with data(unpadded_plaintext).ascii_encipher(key, salt=salt) as enciphering:
+    encipher = data(unpadded_plaintext).ascii_encipher
+    with encipher(key, salt=salt, validator=validator) as enciphering:
         ciphertext = enciphering.list()
         assert len(ciphertext) == 1
         assert ciphertext[0].bit_length() <= 2048
-    with unpack(ciphertext).ascii_decipher(key, salt=salt) as deciphering:
+
+    validator = StreamHMAC(key, salt=salt, pid=pid).for_decryption()
+    decipher = unpack(ciphertext).ascii_decipher
+    with decipher(key, salt=salt, validator=validator) as deciphering:
         deciphered = deciphering.join()
         assert deciphered != unpadded_plaintext
         assert deciphered[246:] == unpadded_plaintext
